<?php
/*
    author: Matheus P. Alves Nogueira
    date: 2017
    e-mail: matheus.nogueira@teknisa.com
*/

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Utilities;

use Teknisa\MAN\Services\ConsolidacaoVendasService;

class ConsolidacaoVendas extends CrudWithParameterBag {

    protected $dataSourceName = "";
    protected $parameterBag;
    protected $utilService;
    protected $parameterBagColumns = array();

    protected $consolidationFileLogService;
    protected $consolidacaoVendasService;

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, ConsolidacaoVendasService $consolidacaoVendasService, Utilities\Util $utilService ) {
        ini_set('max_execution_time', 1800);
        $this->consolidacaoVendasService = $consolidacaoVendasService;
        $this->consolidationFileLogService = $this->consolidacaoVendasService->getLogService();
        $this->utilService      = $utilService;
        parent::__construct($dataSourceManager, $parameterBag);
    }

    public function find(Request\Filter $request, Response $response){
        $dataSourceName            = $this->dataSourceName = 'vnd05400_consolidacaodevendas';
        $conditions = $this->utilService->getParamsAsArray($request);
        
        if(array_key_exists('TODASNF', $conditions) && $conditions['TODASNF'] == "") {
            $conditions['TODASNF'] = 'S';
        }
        
        if(!array_key_exists('DTVENDA', $conditions)) {
            $dataSourceFilter = array();
            $likeableConditions = $this->utilService->getLikeConditions($request);
            
            foreach ($likeableConditions as $condition) {
    			if(array_key_exists($condition['columnName'], $conditions)) {
    				 unset($conditions[$condition['columnName']]);
    			}
    		}
    		
            $conditions['CDCAIXA'] = implode('_', $conditions['CDCAIXA']);
            
            if(!empty($conditions['NRSEQVENDA'])){
                $conditions['NRSEQVENDA'] = implode('_', $conditions['NRSEQVENDA']);    
            }
            
            if(!empty($conditions['CDCLIENTE'])){
                $conditions['CDCLIENTE'] = implode('_', $conditions['CDCLIENTE']);    
            }
            
            if(!empty($conditions['CDTIPOCONS'])){
                $conditions['CDTIPOCONS'] = implode('_', $conditions['CDTIPOCONS']);    
            }
            
            if(!empty($conditions['CDCCUSCLIE'])){
                $conditions['CDCCUSCLIE'] = implode('_', $conditions['CDCCUSCLIE']);    
            }
            
            if(!empty($conditions['CDPRODUTO'])){
                $conditions['CDPRODUTO'] = implode('_', $conditions['CDPRODUTO']);    
            }
            
            $newRequest = $this->utilService->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions);
            foreach ($likeableConditions as $condition) {
                    $newRequest->getFilterCriteria()->addCondition($condition['columnName'], $condition['operator'], $condition['value']);
    		}
    		
            $this->parameterBagColumns = array("CDFILIAL","CDLOJA","DTINICIAL","DTFINAL","CDCAIXA","TCDCAIXA","CDCLIENTE","TCDCLIENTE","NRSEQVENDA","TODASVND","CDTIPOCONS","TCDTIPOCONS","CDCCUSCLIE","TCDCCUSCLIE","CDPRODUTO","TCDPRODUTO","TODASNF","NRORG");
            parent::find($newRequest, $response);
        }
    }

    public function findVendas(Request\Filter $request, Response $response){
        $dataSourceName            = $this->dataSourceName = 'vnd05400_consolidacaodevendas_nrseqvenda';
        $conditions = $this->utilService->getParamsAsArray($request);
        $dataSourceFilter = array();
        $likeableConditions = $this->utilService->getLikeConditions($request);
        foreach ($likeableConditions as $condition) {
			if(array_key_exists($condition['columnName'], $conditions)) {
				 unset($conditions[$condition['columnName']]);
			}
			if(array_key_exists(('T'.$condition['columnName']), $conditions)){
			    $conditions['T'.$condition['columnName']] =  NULL;
			}
		}

		if(!empty($conditions['CDCAIXA'])) {
            $conditions['CDCAIXA'] = implode('_', $conditions['CDCAIXA']);
            $conditions['TCDCAIXA'] = '';
		} else {
            $conditions['CDCAIXA'] = '';
            $conditions['TCDCAIXA'] = 'T';
		}
		
        $newRequest = $this->utilService->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions);
        foreach ($likeableConditions as $condition) {
                $newRequest->getFilterCriteria()->addCondition($condition['columnName'], $condition['operator'], $condition['value']);
		}
        $this->parameterBagColumns = array("DTFINAL","DTINICIAL","TCDCAIXA","CDCAIXA","CDFILIAL","CDLOJA","NRORG");
        parent::find($newRequest, $response);
    }

    public function findCentroCusto(Request\Filter $request, Response $response){
        $this->dataSourceName = 'vnd05400_consolidacaodevendas_cdccusclie';
        $this->parameterBagColumns = array("CDCLIENTE","TCDCLIENTE","NRORG");
         
        $conditions = $this->utilService->getParamsAsArray($request);
        $dataSourceFilter = array();
        $likeableConditions = $this->utilService->getLikeConditions($request);
        
        foreach ($likeableConditions as $condition) {
			if(array_key_exists($condition['columnName'], $conditions)) {
				 unset($conditions[$condition['columnName']]);
			}
			if(array_key_exists(('T'.$condition['columnName']), $conditions)){
			    $conditions['T'.$condition['columnName']] =  NULL;
			}
		}

		if(!empty($conditions['CDCLIENTE'])) {
            $conditions['CDCLIENTE'] = implode('_', $conditions['CDCLIENTE']);
            $conditions['TCDCLIENTE'] = '';
		} else {
            $conditions['CDCLIENTE'] = '';
            $conditions['TCDCLIENTE'] = 'T';
		}
		
        $newRequest = $this->utilService->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions);
        foreach ($likeableConditions as $condition) {
            $newRequest->getFilterCriteria()->addCondition($condition['columnName'], $condition['operator'], $condition['value']);
		}
        parent::find($newRequest, $response);
    }

    public function findProduto(Request\Filter $request, Response $response){
        $this->dataSourceName = 'vnd05400_consolidacaodevendas_cdproduto';
        $this->parameterBagColumns = array(
            "CDFILIAL",
            "CDLOJA",
            "CDCAIXA",
            "TCDCAIXA",
            "DTINICIAL",
            "DTFINAL",
            "NRSEQVENDA",
            "TODASVND",
            "NRORG"
        );
        $conditions = $this->utilService->getParamsAsArray($request);
        $dataSourceFilter = array();
        $likeableConditions = $this->utilService->getLikeConditions($request);
        
        foreach ($likeableConditions as $condition) {
			if(array_key_exists($condition['columnName'], $conditions)) {
				 unset($conditions[$condition['columnName']]);
			}
			if(array_key_exists(('T'.$condition['columnName']), $conditions)){
			    $conditions['T'.$condition['columnName']] =  NULL;
			}
		}

		if(!empty($conditions['CDCAIXA'])) {
            $conditions['CDCAIXA'] = implode('_', $conditions['CDCAIXA']);
            $conditions['TCDCAIXA'] = '';
		} else {
            $conditions['CDCAIXA'] = '';
            $conditions['TCDCAIXA'] = 'T';
		}
		
		if(!empty($conditions['NRSEQVENDA'])) {
            $conditions['NRSEQVENDA'] = implode('_', $conditions['NRSEQVENDA']);
            $conditions['TODASVND'] = '';
		} else {
            $conditions['NRSEQVENDA'] = '';
            $conditions['TODASVND'] = 'S';
		}
		
        $newRequest = $this->utilService->createNewZeedhiFilterRequestObject($request, $this->dataSourceName, $conditions);
        foreach ($likeableConditions as $condition) {
            $newRequest->getFilterCriteria()->addCondition($condition['columnName'], $condition['operator'], $condition['value']);
		}
        parent::find($newRequest, $response);
    }

    public function mountFind($request, $response, $dataSource){
    // método usado para sobrescrever o CrudWithParameterBag por causa de tratamentos adicionais
    // busca dados dinamicamente
        $filterCriteria = $request->getFilterCriteria();

        $this->consolidacaoVendasService->setParameterBagColumns($this->parameterBagColumns, $filterCriteria->getConditions());
        $this->consolidacaoVendasService->setParameterBagValues($this->parameterBag, $filterCriteria->getConditions());
        $this->consolidacaoVendasService->setDataSourceName($this->dataSourceName, $dataSource);
        $newRequest = $this->consolidacaoVendasService->newFilter($request, $this->dataSourceName);

        parent::find($newRequest, $response);
    }

    public function getParavendParams(Request\Row $request, Response $response){
        $row = $request->getRow();
        $this->dataSourceName = 'getParavendParams';

        try {
            $paravendParams = $this->consolidacaoVendasService->getParavendParams($row['CDFILIAL'], $row['NRORG']);
            self::responseFrontEnd($response, true, $paravendParams);
        } catch (Exception $e) {
            self::responseFrontEnd($response, false, $e);
        }
    }
    
    public function dateRestriction(Request\Row $request, Response $response){
        $row = $request->getRow();
        $this->dataSourceName = 'dateRestriction';
        $dateRestriction = null;
        try {
            $dateRestriction = $this->consolidacaoVendasService->dateRestriction($row['filterQuery'], $row['selectQuery']);
            self::responseFrontEnd($response, true, $dateRestriction);
        } catch (Exception $e) {
            self::responseFrontEnd($response, false, $e);
        }
    }
    
    public function groupConsolidation(Request\Row $request, Response $response){
        // agrupa consolidação
        $row = $request->getRow();
        $this->dataSourceName = 'groupConsolidation';
        
        $products     = $row['products'];
        $filterParams = $row['filterParams'];
        $filialParams = $row['filialParams'];
        $lojaParams   = $row['lojaParams'];
        
        $paramEstoque = array();
        $groupConsolidation = array();
        
        try {
            $paramEstoque = $this->consolidacaoVendasService->getParamEstoque($filterParams['NRORG']);
            $this->consolidationFileLogService->saveProcessLog('okStockParams');
            
            // testa consolidação por data da venda
            $consolidationForSaleDate = $this->consolidacaoVendasService->consolidationForSaleDate($products, $filterParams, $filialParams, $paramEstoque);
            $this->consolidationFileLogService->saveProcessLog('okConsolidationPerDateTest');
            
            if ($consolidationForSaleDate['status']) {
                // reúne produtos da consolidação com as observações
                $this->consolidationFileLogService->saveProcessLog('okProductsDoHaveObs');
                $groupConsolidation = $this->consolidacaoVendasService->groupConsolidation($filterParams, $filialParams, $paramEstoque, $lojaParams);
                
                $this->consolidationFileLogService->saveProcessLog('okGroupedProductsWithObs');
                self::responseFrontEnd($response, true, $groupConsolidation);
            } else {
                $this->consolidationFileLogService->saveProcessLog('okProductsDoNotHaveObs');
                self::responseFrontEnd($response, true, $consolidationForSaleDate);
            }
        } catch (Exception $e) {
            $this->consolidationFileLogService->saveExceptionLog('exceptionIntro', $e);
            self::responseFrontEnd($response, false, $e);
        }
    }
    
    public function groupProducts(Request\Row $request, Response $response) {
        // agrupa produtos da consolidacao 
        $this->consolidationFileLogService->saveProcessLog('okGroupProducts');
        $this->dataSourceName = 'groupConsolidation';
        $row = $request->getRow();
        $groupConsolidation = $this->consolidacaoVendasService->agruparProdutosConsolidacao($row[1], $row[2], $row[3]);
        $this->consolidationFileLogService->saveProcessLog('okProductsGroupedWithSuccess');
        self::responseFrontEnd($response, true, $groupConsolidation);
    }
    
    public function insertConsolidation(Request\Row $request, Response $response){
        // insere consolidação no banco
        $row = $request->getRow();
        $this->dataSourceName = 'insertConsolidation';
        
        $data         = $row['toConsolidate'];
        $gridItens    = $row['gridItens'];
        $filterParams = $row['filterParams'];
        $filialParams = $row['filialParams'];
        
        if(empty($filterParams['CDSETOR']))
            $filterParams['CDSETOR'] = null;    
        
        $insertConsolidation = array();
        
        try {
            $insertConsolidation = $this->consolidacaoVendasService->insertConsolidation($data, $gridItens, $filterParams, $filialParams);
            self::responseFrontEnd($response, true, $insertConsolidation);
        } catch (Exception $e) {
            self::responseFrontEnd($response, false, $e);
        }
    }
    
    public function responseFrontEnd(&$response, $type, $data) {
        if ($type)
            $response->addDataSet(new DataSet($this->dataSourceName, $data));
        else
            $response->setError(new Response\Error($data->getMessage(), $data->getCode()));
    }
    
    public function getLikeConditions($request) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $result = array();
        foreach ($conditions as $condition) {
            if($condition['operator'] === 'LIKE_ALL' )
                array_push($result, $condition);
        }
        return $result;
    }
    
    public function habilitaPesqProd(Request\Row $request, Response $response) {
        $row = $request->getRow();
        
        $params = array(
            ':CDFILIAL' => $row['CDFILIAL'],
            ':NRORG'    => $row['NRORG']
        );
        if(!empty($row['CDFILIAL']) && !empty($row['NRORG'])){
            $habilitaProd[0] = $this->consolidacaoVendasService->habilitaPesqProd($params)[0]['IDGERANFVENPROD'];
        }else{
            $habilitaProd[0] = 'N';
        }
        
        $response->addDataSet(new DataSet('habilitaProd', $habilitaProd));
    }
}